//
// Created by Felix on 24-4-26.
//

#ifndef GAZE_AUTHINTERCEPTOR_HPP
#define GAZE_AUTHINTERCEPTOR_HPP

#include "oatpp/web/server/interceptor/RequestInterceptor.hpp"
#include "oatpp/web/server/HttpRouter.hpp"
#include "oatpp/web/server/handler/AuthorizationHandler.hpp"

#include "handler/AuthHandler.hpp"

class AuthInterceptor : public oatpp::web::server::interceptor::RequestInterceptor {
private:
    AuthHandler iAuthHandler;
    oatpp::web::server::HttpRouterTemplate<bool> authEndpoints;
public:
    AuthInterceptor() {
        authEndpoints.route("POST", "/hello", false);
        authEndpoints.route("POST", "/api/auth/sign_in", false);
        authEndpoints.route("GET", "/api/auth/sign_out", false);

        authEndpoints.route("GET", "/swagger/*", false);
        authEndpoints.route("GET", "/api-docs/oas-3.0.0.json", false);
    }

    std::shared_ptr<AuthInterceptor::OutgoingResponse> intercept(
            const std::shared_ptr<IncomingRequest> &request
    ) override {
        auto r = authEndpoints.getRoute(
                request->getStartingLine().method, request->getStartingLine().path
        );
        if (r && !r.getEndpoint()) {
            return nullptr; // Continue without Authorization
        }

        auto authHeader = request->getHeader(oatpp::web::protocol::http::Header::AUTHORIZATION);
        auto authObject = std::static_pointer_cast<AuthHandler::Payload>(iAuthHandler.authorize(authHeader));
        if (authObject && authObject->userId) {
            request->putBundleData("userId", authObject->userId);
            return nullptr; // Continue - token is valid.
        }

        throw oatpp::web::protocol::http::HttpError(oatpp::web::protocol::http::Status::CODE_401, "Unauthorized", {});
    }
};

#endif //GAZE_AUTHINTERCEPTOR_HPP
